from armet.connectors.flask import resources as flask_resources
from armet.connectors.sqlalchemy import resources as sqlalchemy_resources
from armet import utils
from alchemist import db

__all__ = [
    'Resource',
    'ResourceOptions',
    'ModelResource',
]


class ResourceOptions(object):

    def __init__(self, meta, name, bases):
        #! SQLAlchemy session used to perform operations on the models.
        self.Session = meta.get('Session')
        if not self.Session:
            # Default this to an alchemist-provided session.
            self.Session = db.Session


class Resource(flask_resources.Resource):

    def route(self, *args, **kwargs):
        # Establish a session.
        self.session = session = db.Session()

        # Bind the session to the thread-local store.
        db._local.instance = self.session

        try:
            # Continue on with the cycle.
            result = utils.super(Resource, self).route(*args, **kwargs)

            # Commit the session.
            session.commit()

            # Return the result.
            return result

        except:
            # Something occurred; rollback the session.
            session.rollback()

            # Re-raise the exception.
            raise

        finally:
            # Close the session.
            session.close()

            # Revoke the thread-local session.
            del db._local.instance


class ModelResource(sqlalchemy_resources.ModelResource):
    pass



# -*- coding: utf-8 -*-
import types
import sys
import threading


class Module(types.ModuleType):

    #! Locally available session object.
    _local = threading.local()

    @property
    def session(self):
        # Check for an existing session.
        if not hasattr(self._local, 'instance'):
            # No session available; create one.
            self._local.instance = self.Session()

        # Return the available session.
        return self._local.instance

    @property
    def Session(self):
        # Save these for reference.
        from sqlalchemy import orm

        # Construct an inner class to late-bind configuration.
        class Session(orm.Session):

            def __init__(self, *args, **kwargs):
                from alchemist import application
                from alchemist.db.query import Query

                # Default the bind and query class.
                kwargs.setdefault('bind', application.databases['default'])
                kwargs.setdefault('query_cls', Query)

                # Continue the initialization.
                super().__init__(*args, **kwargs)

            def __repr__(self):
                return '<alchemist.db.Session(bind=%r)>' % self.bind

        # Return the inner class.
        return Session

# Update the inner module with the actual contents.
instance = Module(__name__)
instance.__dict__.update(sys.modules[__name__].__dict__)

# Store the module wrapper
sys.modules[__name__] = instance
